
from xmmsvalue import VALUE_TYPE_NONE
from xmmsvalue import VALUE_TYPE_ERROR
from xmmsvalue import VALUE_TYPE_INT64 as VALUE_TYPE_INT32
from xmmsvalue import VALUE_TYPE_INT64
from xmmsvalue import VALUE_TYPE_STRING
from xmmsvalue import VALUE_TYPE_COLL
from xmmsvalue import VALUE_TYPE_BIN
from xmmsvalue import VALUE_TYPE_LIST
from xmmsvalue import VALUE_TYPE_DICT

from xmmsvalue import COLLECTION_TYPE_REFERENCE
from xmmsvalue import COLLECTION_TYPE_UNIVERSE
from xmmsvalue import COLLECTION_TYPE_UNION
from xmmsvalue import COLLECTION_TYPE_INTERSECTION
from xmmsvalue import COLLECTION_TYPE_COMPLEMENT
from xmmsvalue import COLLECTION_TYPE_HAS
from xmmsvalue import COLLECTION_TYPE_MATCH
from xmmsvalue import COLLECTION_TYPE_TOKEN
from xmmsvalue import COLLECTION_TYPE_EQUALS
from xmmsvalue import COLLECTION_TYPE_NOTEQUAL
from xmmsvalue import COLLECTION_TYPE_SMALLER
from xmmsvalue import COLLECTION_TYPE_SMALLEREQ
from xmmsvalue import COLLECTION_TYPE_GREATER
from xmmsvalue import COLLECTION_TYPE_GREATEREQ
from xmmsvalue import COLLECTION_TYPE_ORDER
from xmmsvalue import COLLECTION_TYPE_LIMIT
from xmmsvalue import COLLECTION_TYPE_MEDIASET
from xmmsvalue import COLLECTION_TYPE_IDLIST


from xmmsapi import PLAYBACK_STATUS_STOP
from xmmsapi import PLAYBACK_STATUS_PLAY
from xmmsapi import PLAYBACK_STATUS_PAUSE

from xmmsapi import PLAYLIST_CHANGED_ADD
from xmmsapi import PLAYLIST_CHANGED_INSERT
from xmmsapi import PLAYLIST_CHANGED_SHUFFLE
from xmmsapi import PLAYLIST_CHANGED_REMOVE
from xmmsapi import PLAYLIST_CHANGED_CLEAR
from xmmsapi import PLAYLIST_CHANGED_MOVE
from xmmsapi import PLAYLIST_CHANGED_SORT
from xmmsapi import PLAYLIST_CHANGED_UPDATE

from xmmsapi import PLUGIN_TYPE_ALL
from xmmsapi import PLUGIN_TYPE_XFORM
from xmmsapi import PLUGIN_TYPE_OUTPUT

from xmmsapi import MEDIALIB_ENTRY_STATUS_NEW
from xmmsapi import MEDIALIB_ENTRY_STATUS_OK
from xmmsapi import MEDIALIB_ENTRY_STATUS_RESOLVING
from xmmsapi import MEDIALIB_ENTRY_STATUS_NOT_AVAILABLE
from xmmsapi import MEDIALIB_ENTRY_STATUS_REHASH

from xmmsapi import COLLECTION_CHANGED_ADD
from xmmsapi import COLLECTION_CHANGED_UPDATE
from xmmsapi import COLLECTION_CHANGED_RENAME
from xmmsapi import COLLECTION_CHANGED_REMOVE

from xmmsapi import PLAYBACK_SEEK_SET
from xmmsapi import PLAYBACK_SEEK_CUR

from xmmsapi import COLLECTION_NS_COLLECTIONS
from xmmsapi import COLLECTION_NS_PLAYLISTS
from xmmsapi import COLLECTION_NS_ALL
